function [v,p]=costoa(D,H,l,pre)
err=1
[m,n]=size(H)
v=-%inf*ones(n,1)
id=(1+l)*speye(n,n)
iter=1
while err>pre 
  iter=iter+1
  [h,p,c]=amax(D,H*v)
  w=lusolve(id-p*H,c)
  err=norm(v-w,'inf')
  v=w
end
  
function [v0,q]=costoe(D,H,pre)
[v,p]=costoa(D,H,0.011,pre)
[w,q]=costoa(D,H,0.001,pre)
v0=0.0011*(w-v)

function [v,p]=costo(D,H,l)
//
// Commande stochastique
//
if l>0 then
  [v,p]=costoa(D,H,l,0.001)
  v=l*v
else
  [v,p]=costoe(D,H,0.001)
end

function [W,f,iter]=WardropC(H,d,a,pre)
//
// tentative wardrop cout cubique
// pour memoire
//
pen=0.01
ci=10
C=0.1
err=100
[m,n]=size(H)
V=ones(m,1)
un=ones(n,1)
nd=size(d,2)
D=d*ones(nd,1)
Q=diag(sparse(un*C))
qs=zeros(n,1)
QS=qs 
q=sparse(ones(n,nd)*ci)
iter=1
while err>pre 
  iter=iter+1
  DD=nd*D+H*Q*qs
  W=lusolve(H*Q*H'+a*speye(m,m),DD);
  QS=(H'*W-qs)/nd
  Q=diag(sparse(max(0.001,C*real(un./sqrt(QS)))))
  qs=spzeros(n,1)
  for i=1:nd
    di=d(:,i)+H*diag(QS)*q(:,i)
    w=lusolve(H*sparse(diag(q(:,i)))*H'+a*speye(m,m),di);
    qqq=H'*w-QS
    qs=qs+qqq
     q(:,i)=sparse(max(pen,ci*real(un./sqrt(qqq))))
  end 
  err=norm(V-W,'inf')
  disp(iter,err)
  V=W
end
f=Q

function [v,Q,iter]=fluideHow(H,d,a,pre)
err=1
[m,n]=size(H)
v=ones(m,1)
un=ones(n,1)
q=diag(sparse(un/2.0))
iter=1
while err>pre 
  iter=iter+1
  disp(iter)
  w=lusolve(H*q*H'+a*speye(m,m),d);
  q=diag(sparse(un./sqrt(abs(H'*w))))
  err=norm(v-w,'inf')
  disp(err)
  v=w
end
Q=q*H'*w

function [v,Q,iter]=fluide(H,d,r,a,pre)
//
// Reseau d'eau
//
err=1
[m,n]=size(H)
v=ones(m,1)
un=ones(n,1)
R=diag(sparse(r))
q1=r
q=R
iter=0
printf("  ")
printf("=====================")
printf("Fluide/C/%5d/%5d/",m,n)
printf("=====================")
printf("  ")
printf("------------------")
printf("| iter |  erreur |")
printf("------------------")
while err>pre 
  iter=iter+1
  d1=2*d-H*q1
  w=lusolve(H*q*H'+a*speye(m,m),d1);
  q1=(q*H'*w+q1)/2
  q=R*diag(sparse(un./abs(q1)))
  err=norm(v-w,'inf')
  printf("|  %2d  | %2.1e |",iter,err)
  v=w
end
Q=q1

function [v,Q]=ReseauDioRst(H,d,r,a,pre,niter)
//
// Reseau de resistances laissant passer le courant
// dans une seule direction
//
err=1
[m,n]=size(H)
v=ones(m,1)
un=ones(n,1)
R=diag(sparse(r))
q1=r
q=R
iter=0
printf("  ")
printf("=====================")
printf("Wardrop1/Q/%4d/%4d/",m,n)
printf("=====================")
printf("  ")
printf("------------------")
printf("| iter |  erreur |")
printf("------------------")
while err>pre & iter<niter
  iter=iter+1
//  pause
  w=lusolve(H*q*H'+a*speye(m,m),d);
  q1=H'*w
  q=R*diag(sparse(max(0,sign(q1))))
  err=norm(v-w,'inf')
  printf("|  %2d  | %2.1e |",iter,err)
  v=w
end
Q=R*max(0,q1)

function [F,q,w]=Wardrop(H,d,l,r,a,pre,niter)
//function [F,q,ww]=Wardrop(H,d,l,r,a,k,pre,niter)
//
// Equilibre de Wardrop temps terme lineaire
// plus un terme quadratique
//
k=1;
R=diag(sparse(r));
err=1;
[m,n]=size(H);
un=ones(n,1);
unm=ones(m,1);
nd=size(d,2);
und=ones(nd,1);
w=ones(m,nd);
D=full(d*und);
Q=R;
F=r;
sq=diag(sparse(un));
q=r*ones(1,nd);
//
// Resolution du probleme agrege
//
iter=0;
printf("  ")
printf("==========================\n")
printf("Wardropn/Q/%4d/%4d/%4d/\n",m,n,nd)
printf("==========================\n")
printf("  ")
printf("------------------\n")
printf("| iter |  erreur |\n")
printf("------------------\n")
iter=0;
ac=a;
// wold=0;
ww=0;
while niter>iter & err>pre
  ac=ac/k;
  iter=iter+1;
  DL=D*sqrt(nd)+H*sq*R*l+a*ww
  ww=lusolve(H*sq*R*H'+a*speye(m,m),DL);
//  printf("%2.1e|%2.1e",max(ww-wold),min(ww-wold));
  f=max(0,full(R*(H'*ww-l)));
  sq=diag(sparse(sign(f)));
  err=norm(f-F,'inf')
  printf("|  %2d  | %2.1e |\n",iter,err)
  F=f
//  wold=ww
end
errV=norm(diag(sparse(sign(F)))*(R*(H'*ww-l)-F))/(norm(F)+1.e-10);
errQ=norm(H*F-sqrt(nd)*D)/(sqrt(nd)*norm(D)+1.e-10);
printf("===========================\n");
printf("|errV=%2.1e|errQ=%2.1e|\n",errV,errQ);
printf("===========================\n");
//
// Iteration pour assurer la positivite
// des flots individuels.
//
iter=0;
err=1;
ac=a;
for i=1:nd
  q(:,i)=f/nd;
  w(:,i)=ww;
end
while niter>iter & err>pre
  ac=ac/k;
  iter=iter+1;
//  wold=ww; 
  for i=1:nd
    qi=q(:,i);
    fi=f-qi;
    admit=diag(sparse(sign(qi)));
    iteri=0;
    erri=1;
//    w(:,i)=unm;
    while 8>iteri & erri>pre
      iteri=iteri+1;
      di=d(:,i)+H*admit*(fi+R*l)+a*w(:,i);
      ww=lusolve(H*admit*R*H'+a*speye(m,m),full(di));
      qi=max(full(R*(H'*ww-l)-fi),0);
      admit=diag(sparse(sign(qi)));
//      erri=norm(ww-wold);
      erri=norm(ww-w(:,i));
//      printf("|  %2d  | %2.1e |\n",iteri,erri) //
      w(:,i)=ww;
//      wold=ww
    end
    q(:,i)=qi;
    f=fi+qi;
  end 
  f=q*und
  err=norm(f-F,'inf')
//printf("------------------\n")
printf("|  %2d  | %8.7e | %2.1e |\n",iter,time,cout,err)
//printf("------------------\n")
F=f
end
errV=0;
errQ=0;
//admit=diag(sparse(sign(F)));
//W=lusolve(H*admit*R*H'+ac*speye(m,m),full(H*admit*(F+R*l)));
//W=W-min(W);
for i=1:nd
  errV=errV+norm(diag(sparse(sign(q(:,i))))*(R*(H'*w(:,i)-l)-F));
//  errV=errV+norm(diag(sparse(sign(q(:,i))))*(R*(H'*ww-l)-F));
   errQ=max(errQ,norm(H*q(:,i)-d(:,i),"inf"));
end
printf("===========================\n");
printf("|errV=%2.1e|errQ=%2.1e|\n",errV/(nd*norm(F)+1.e-10),errQ/(1.e-10+norm(q,"inf")));
printf("===========================\n");


