function [a]=%zeros(n,m)
a=sparse(%0*%ones(n,m))
//
//Copyright INRIA
//
//============
// MATRIX STAR 
//============
//
//function [sa]=%talg_5(a)
//sa=star(a);
//
//=================================================================
// SYSTEMES LINEAIRES MAXPLUS (CREATION ET OPERATIONS ELEMENTAIRES)
//================================================================
//
function [sl]=mpsyslin(a,b,c,d,x0)
//
[lhs,rhs]=argn(0)
//
if rhs==1 then [m,n]=size(a)
  sl=tlist(['mpls','A','B','C','D','X0'],%zeros(n,n),..
      %eye(n,n),#(a),%zeros(n,n),%zeros(n,1))
else 
  if rhs==2 then error('mpsyslin: not enough arguments')  
  elseif rhs>2 then 
    //mpsyslin(A,B,C [,D [X0]])
    if type(a)<>[1,5,257,261] then
      error('mpsyslin : A must be a square matrix of numbers')
    end
    [ma,na]=size(a);
    if ma<>na then 
      error('mpsyslin : A must be a square matrix of numbers')
    end
    if type(b)<>[1,5,257,261] then
      error('mpsyslin : B must be a  matrix of numbers')
    end
    [mb,nb]=size(b);
    if na<>mb&mb<>0 then 
      error('mpsyslin : row dimension of B do not agree dimensions of A')
    end
    if type(c)<>[1,5,257,261] then
	error('mpsyslin : C must be a  matrix of numbers')
    end
    [mc,nc]=size(c);
    if na<>nc&nc<>0 then 
	error('mpsyslin : column dimension of C do not agree dimensions of A')
    end
    if rhs<5 then
	  x0=%zeros(na,1)
    else
          if type(x0)<>[1,5,257,261] then
	    error('mpsyslin : X0 must be a vector of numbers')
           end
	  [mx,nx]=size(x0);
          if mx<>na|nx<>min(na,1) then 
	    error('mpsyslin : dimensions of X0 do not agree')
           end
    end
    if rhs<4  then
	    d=%zeros(na,na)
     else
	    if type(d)<>[1,5,257,261] then
	      error('mpsyslin : D must be a  matrix of numbers or polynomials')
	    end
	    [md,nd]=size(d);
	    if md<>na|nd<>na then 
	      error('syslin : column dimension of D do not agree dimensions of A')
	    end
     end
	    sl=tlist(['mpls','A','B','C','D','X0'],#(a),#(b),#(c),#(d),#(x0))
  end 
end
	
//
//=================
//MISE EN PARALLELE
//=================
//
function [s]=%mpls_a_mpls(s1,s2)
//
//  Ss=S1+S2 in state-space
//
//[s1 s2]=sysconv(s1,s2)
[a1,b1,c1,d1,x1]=s1(2:6);
[a2,b2,c2,d2,x2]=s2(2:6);
[n1,n1]=size(a1);
[n2,n2]=size(a2);
a1=[a1,%zeros(n1,n2);%zeros(n2,n1),a2];
d1=[d1,%zeros(n1,n2);%zeros(n2,n1),d2];
s=tlist(['mpls','A','B','C','D','X0'],a1,[b1;b2],[c1,c2],d1,[x1;x2]);
//
//=============
//MISE EN SERIE
//=============
//
function [s]=%mpls_m_mpls(s2,s1)
//
// computes S1*S2 in state-space form.
//
[a1,b1,c1,d1,x1]=s1(2:6);
[a2,b2,c2,d2,x2]=s2(2:6);
[n1,n1]=size(a1);
[n2,n2]=size(a2);
[n1,nb1]=size(b1);
[nc2,n2]=size(c2);
a1=[a1,%zeros(n1,n2);%zeros(n2,n1),a2];
d1=[d1,%zeros(n1,n2);b2*c1,d2];
b1=[b1;%zeros(n2,nb1)];
c1=[%zeros(nc2,n1),c2];    
s=tlist(['mpls','A','B','C','D','X0'],a1,b1,c1,d1,[x1;x2]);
//
//=======================================
//MULTIPLICATION PAR UNE MATRICE A GAUCHE
//=======================================
//
function [s]=%talg_m_mpls(m,s)
//
// computes m*S in state-space form.
//
[a,b,c,d,x]=s(2:6);
s=tlist(['mpls','A','B','C','D','X0'],a,b,m*c,d,x);
//
//idem avec un scalaire standard
//
function [s]=%s_m_mpls(m,s)
[a,b,c,d,x]=s(2:6);
s=tlist(['mpls','A','B','C','D','X0'],a,b,#(m)*c,d,x);
//
//=======================================
//MULTIPLICATION PAR UNE MATRICE A DROITE
//=======================================
//
function [s]=%mpls_m_talg(s,m)
//
// computes S*m in state-space form.
//
[a,b,c,d,x]=s(2:6);
s=tlist(['mpls','A','B','C','D','X0'],a,b*m,c,d,x);
//
// idem avec un scalaire standard
//
function [s]=%mpls_m_s(s,m)
[a,b,c,d,x]=s(2:6);
s=tlist(['mpls','A','B','C','D','X0'],a,b*m,c,d,x);
//
//========
//FEEDBACK
//========
//
function [s]=%mpls_v_mpls(s1,s2)
//
// computes star(S1*S2)*S1 in state-space form.
//
[a1,b1,c1,d1,x1]=s1(2:6);
[a2,b2,c2,d2,x2]=s2(2:6);
[n1,n1]=size(a1);
[n2,n2]=size(a2);
[n1,nb1]=size(b1);
[nc1,n1]=size(c1);
a1=[a1,%zeros(n1,n2);%zeros(n2,n1),a2];
d1=[d1,b1*c2;b2*c1,d2];
b1=[b1;%zeros(n2,nb1)];
c1=[c1,%zeros(nc1,n2)];    
s=tlist(['mpls','A','B','C','D','X0'],a1,b1,c1,d1,[x1;x2]);
//
//================
// OUTPUT ADDITION
//================
//
function [s]=%mpls_c_mpls(s1,s2)
//
// computes [S1,S2] that is  concatenation of inputs, addidion of outputs
//
[a1,b1,c1,d1,x1]=s1(2:6);
[a2,b2,c2,d2,x2]=s2(2:6);
[n1,n1]=size(a1);
[n2,n2]=size(a2);
[n1,nb1]=size(b1);
[n2,nb2]=size(b2);
a1=[a1,%zeros(n1,n2);%zeros(n2,n1),a2];
d1=[d1,%zeros(n1,n2);%zeros(n2,n1),d2];
b1=[b1,%zeros(n1,nb2);%zeros(n2,nb1),b2];
s=tlist(['mpls','A','B','C','D','X0'],a1,b1,[c1,c2],d1,[x1;x2]);
//
//================
//INPUTS IN COMMON 
//================
//
function [s]=%mpls_f_mpls(s1,s2)
//
// computes [S1;S2] that is :  inputs in common,  concatenation of outputs
//
[a1,b1,c1,d1,x1]=s1(2:6);
[a2,b2,c2,d2,x2]=s2(2:6);
[n1,n1]=size(a1);
[n2,n2]=size(a2);
[nc1,n1]=size(c1);
[nc2,n2]=size(c2);
a1=[a1,%zeros(n1,n2);%zeros(n2,n1),a2];
d1=[d1,%zeros(n1,n2);%zeros(n2,n1),d2];
c1=[c1,%zeros(nc1,n2);%zeros(nc2,n1),c2];
s=tlist(['mpls','A','B','C','D','X0'],a1,[b1;b2],c1,d1,[x1;x2]);
//
//==============
// CONCATENATION
//==============
//
function [s]=%mpls_g_mpls(s1,s2)
//
// computes S1|S2 that is : concatenation of systems
//
[a1,b1,c1,d1,x1]=s1(2:6);
[a2,b2,c2,d2,x2]=s2(2:6);
[n1,n1]=size(a1);
[n2,n2]=size(a2);
[n1,nb1]=size(b1);
[n2,nb2]=size(b2);
[nc1,n1]=size(c1);
[nc2,n2]=size(c2);
a1=[a1,%zeros(n1,n2);%zeros(n2,n1),a2];
d1=[d1,%zeros(n1,n2);%zeros(n2,n1),d2];
b1=[b1,%zeros(n1,nb2);%zeros(n2,nb1),b2];
c1=[c1,%zeros(nc1,n2);%zeros(nc2,n1),c2];
s=tlist(['mpls','A','B','C','D','X0'],a1,b1,c1,d1,[x1;x2]);
//
//===========
// EXTRACTION
//===========
//
function [s]=%mpls_e(i,j,f)
//
// s= f(i,j)
//
[l,a,b,c,d,x]=f(1:6)
s=tlist(l,a,b(:,j),c(i,:),d,x)
//
//======
// SHIFT
//======
//
function [s]=shift(n,t)
//
// system : n-shift in event numbers  and t-shift in dates
//
if n<1 then error( "shift: n must be larger or equal to 1") 
end  
a=sparse(diag(%ones(1,n),1));
b=[%zeros(n,1);sparse(%1)];
c=[sparse(#(t)),%zeros(1,n)];
s=mpsyslin(a,b,c);
//
//========================================
//TAILLE SIZE D'UN SYST. MAX-PLUS LINEAIRE
//========================================
// pas de surcharge a cause de la presence d'un bug
//
function [nx,nu,ny]=mplssize(x)
//
[b,c]=x(3:4);[nx,nu]=size(b);[ny,nx]=size(c);
//
//==================
// SPARSIFY  A SYSTEM
//==================
//
function [sp]=%mpls_sparse(s)
[l,a,b,c,d,x]=s(1:6)
sp=tlist(l,sparse(a),sparse(b),sparse(c),sparse(d),sparse(x));
//
//==================
// FULLIFY  A SYSTEM
//==================
// usage full(s)
//
function [sp]=%mpls_full(s)
[l,a,b,c,d,x]=s(1:6)
sp=tlist(l,full(a),full(b),full(c),full(d),full(x));
//
//==============================
// EXPLICIT AND SIMPIFIED SYSTEM
//==============================
//
function [sn]=explicit(so)
//
sf=full(so);
[a,b,c,d]=sf(2:5);
ds=star(d); a=ds*a; b=ds*b; 
ac=[a;c]; zerocol=%ones(1,size(ac,1))*ac<>%0;
keep=find(zerocol);
sn=sparse(mpsyslin(ac(keep,keep),b(keep,:),c(:,keep)));
//
//===========
// SIMULATION
//===========
//
function [y]=simul(s,u)
//
// s max-plus linear system
// u max-plus input
// y max-plus output
//
sf=full(s);
[a,b,c,d,x]=sf(2:6);y=[];
for k=1:size(u,2)
  x=a*x+b*u(:,k)
  z=c*x
  y=[y,z]
end;
//
//=====================================
// PRETTY PRINT MAX-PLUS LINEAR SYSTEMS
//=====================================
//
function []=mpssprint(sl,out)
// pretty print of a linear system in state-space form
// sl=(A,B,C,D) syslin list.
[lhs,rhs]=argn(0)
mess='system cannot be displayed in this page'
fil=%f
if rhs==1 then 
  out=%io(2),
else
  if type(out)==10 then
  out=file('open',out,'unknown')
  fil=%t
  end
end

//
// x=dx+ax'+ bu
//
write(out,' ')
sgn='+'
ll=lines();ll=ll(1)
[a,b,c,d]=sl(2:5)
a=full(a)
b=full(b)
c=full(c)
d=full(d)
withd=%ones(1,size(d,1))*d*%ones(size(d,2),1)<>%0
na=size(a,1)
nc=size(c,1)
nb=size(b,2)
if na>0 then
  blank=[];for k=1:na,blank=[blank;'           '],end

  ta=%cv(a);
  tb=%cv(b);
  if withd then td=%cv(d);end
  //
  blank=part(blank,1:4)
  if na==1 then
    if withd then
      t=['    ';'x = ']+[' ';td]+['  ';'x + ']+['  ';ta]+[' ';'x''']
    else
      t=['    ';'x = ']+['  ';ta]+[' ';'x''']
    end
  else
    blank([na/2,na/2+1])=['    ';'x = ']
    if withd then
      t=blank+td
      blank([na/2,na/2+1])=['    ';'x + ']
      t=t+blank+ta;
    else
      t=blank+ta
    end
  end
  if nb>0 then
    if na==1 then
      t=t+['    ';' + ']+[' ';tb]+[' ';'u   ']
    else
      blank([na/2,na/2+1])=['    ';'x''+ ']
      t=t+blank+tb
      t(na/2+1)=t(na/2+1)+'u   '
    end
  end
  n1=maxi(length(t))+1
  //
  n2=maxi(length(t))
  if n2<ll then
    write(out,t)
  else
    if n1<ll,
      if n2-n1<ll then
	write(out,part(t,1:n1)),
	write(out,' ')
	write(out,part(t,n1+1:n2),'(3x,a)')
      else          
	disp('x(n)=Dx(n)+Ax(n-1)+Bu(n)')
	disp(sl('A'),'A='),
	disp(sl('B'),'B='),
	disp(sl('D'),'D='),  // error(mess)
      end ;
    else 
     disp('x(n)=Dx(n)+Ax(n-1)+Bu(n)')
     disp(sl('A'),'A='),
     disp(sl('B'),'B='),
     disp(sl('D'),'D='),  // error(mess)
    end ;
  end ;
end
//
//y = cx 
//
if nc==0 then if fil then file('close',out);end;return,end
write(out,' ')
blank=[];for k=1:nc,blank=[blank;'           '],end
tc=%cv(c);

if na==0 then
  if nc==1 then 
    t='y = '+td+'u'
  else
    blank(nc/2+1)='y = '
    t=blank
    t(nc/2+1)=t(nc/2+1)+'u   '
  end
  n1=maxi(length(t))+1
else
  blank=part(blank,1:4);
  if nc==1 then 
    t='y = '+tc
  else
    blank(nc/2+1)='y = '
    t=blank+tc;
  end
  if nc==1 then 
    t=t+'x   '
  else
    t(nc/2+1)=t(nc/2+1)+'x   '
  end
end
n2=maxi(length(t))
if n2<ll then
  write(out,t)
else
  if n1<ll,
    if n2-n1<ll then
      write(out,part(t,1:n1)),
      write(out,' ')
      write(out,part(t,n1+1:n2),'(3x,a)')
    else 
      disp(sl('C'),'C='), //  error(mess)
    end
  else 
    disp(sl('C'),'C='),  // error(mess)
  end
end
if fil then file('close',out),end

function ta=%cv(x)
[m,n]=size(x);
if m*n==0 then ta=' ',return,end;
frmt=format();frmt=10^frmt(2);
//x=round(frmt*x)/frmt;
t=[];
for k=1:m,t=[t;'|'],end;
ta=t;
for k=1:n,
  aa=string(x(:,k))
  for l=1:m
    if part(aa(l),1)<>'-' then
      aa(l)=' '+aa(l)
    end
  end
  nn=maxi(length(aa));
  aa=part(aa+blank,1:nn);
  ta=ta+aa+part(blank,1)
end;
ta=ta+t;

function []=%mpls_p(x)
mpssprint(x)
