// Copyright INRIA
//insertion

demolist=[];
exec("../loader.sce")

function[s]=%talg_string(n)
s=string(plustimes(n));
if s<>[], s=strsubst(s,'-Inf','@'), end ;                                     
 endfunction;                          


//=======================================
// 1) MAX-PLUS TYPE, SCALARS AND MATRICES
//=======================================
//
// Standard type
a=2
type(a)
// Max-plus type
b=#(3)
type(b)
// Changing type
plustimes(b)
type(ans)
// Max-Plus Zero
%0
type(%0)
// Standard infinity
%inf
type(%inf)
// Max-plus one
%1
// Max-plus operations
b
type(b)
b + %0
b * %1
b + b
b * b
b / b
b & %0
b == b
b <> b
b >= b
b > b
// Operation between objects of different types
b+3
type(ans)
b*3
// Max-plus matrix creation
//
// from a max-plus scalar
c=[b,4;5,6]
type(c)
// from a standard matrix
d=[1,2;3,4]
type(d)
e=#(d)
type(e)
// by extraction
f=e(1,:)
type(f)
// by insertion
e(5,5)=6
type(e)
// Important matrices
%ones(2,5)
%eye(2,5)
g=%zeros(2,5)
// g is a sparse matrix
type(g)
// changing a sparse in a full matrix
full(g)
type(ans)
// changing a full in sparse matrix
%ones(2,5)
sparse(ans)
type(ans)
// Max-plus operations on matrices
c
d
c + d
c * c
c / c
d & c
star(c)
c == c
c <> c
d > c
// Column concatenation
h=[e,e]
// Row concatenation
i=[e;e]
size(i)
// Extraction
i([1,3],:)
// Spectral computation
c
[chi,v]=howard(c)
// chi is the eigenvalue, v is the eigenvector
chi(1)*v==c*v
// The spectral elements give the asymptotic behaviour
x=[%1;%0]
[x,c*x,c*c*x,c*c*c*x,c*c*c*c*x]
//
//=========================================================
// 2) STATE SPACE REPRESENTATION OF MAX-PLUS LINEAR SYSTEMS
//=========================================================
//
// Creation of max-plus dynamical linear systems in implicit state form :
//
//     X(n)=DX(n)+AX(n-1)+BU(n),  Y(n)=CX(n)
//
s1=mpsyslin([1,2;3,4],[0;0],[0,0],%eye(2,2))
s1('D')
s1('X0')
s1=full(s1)
s1('X0')
explicit(s1)
s2=mpsyslin([1,2,3;4,5,6;7,8,9],[0;0;0],[0,0,0],%eye(3,3))
s2=sparse(s2);
//
// System operators have the same syntax as the matrix ones
//
// Diagonal composition
s4=s1|s2
// Parallel composition
s3=s1+s2
// Series composition
s1*s2
// Input in common
[s1;s2]
// Output addition
[s1,s2]
// Feedback composition
s1/.s2
// Extraction 
s4=full(s4)
s4(1,1)
// Simulation
y=simul(s1,[1:10])
plot2d(y')
//
//=====================================
// 3) APPLICATION TO PRODUCTION SYSTEMS
//=====================================
//
// Entering the processing times of a flowshop
//
PT=[#(2),3.9,0.95,1.1,0.7,1.4;
%0,%0,2,1.2,%0,1.7;
3.7,%0,2.2,%0,6.4,%0;
%0,%0,2,%0,1,1;
1.7,3.1,3,%0,1.3,%0;
0.5,3.2,4.3,1.9,1.6,0.4;
1,1,1,1,1,1;
1.5,1.5,1.5,1.2,1.2,1.2]
//
[nmach,npiece]=size(PT)
//
// Entering the machine numbers
//
nm=ones(1,nmach)
//
// Entering the pallet numbers
//
np=ones(1,npiece)
//
//
//-----------------------------------------------
// 3.2) STATE SPACE REPRESENTATION AND SIMULATION 
//-----------------------------------------------
//
// Open-loop flowshop state space 4-uple
//
s=flowshop(PT)
//
// Machine feedback
//
nm
//
fbm=shift(nm(1),0) ;
for i=1:nmach-1, fbm=fbm|shift(nm(i),0) ; end ;
//
// Pallet feedback
//
np
//
fbp=shift(np(1),0);
for i=1:npiece-1, fbp=fbp|shift(np(i),0) ; end ;
fbp
//
//
// Feedback system
//
sb=s/.(fbp|fbm);
//
//  Reducing and putting in explicit form
//
sbs=explicit(sb);
//
// Simulation of the feedback system
//
u=ones(nmach+npiece,1)*(1:100);
y=simul(sbs,u);
y(:,100)'
//
// @@@@@@@@@@@@@
// @           @
// @  THE END  @
// @           @
// @@@@@@@@@@@@@
